package com.screentimetracker.service

import android.app.Service
import android.content.Intent
import android.os.IBinder
import android.app.NotificationManager
import android.content.Context
import android.util.Log
import android.view.WindowManager
import com.screentimetracker.ScreenTimeApplication
import kotlinx.coroutines.*

class ScreenTimeService : Service() {
    companion object {
        private const val TAG = "ScreenTimeService"
        const val ACTION_REQUEST_UPDATE = "com.screentimetracker.REQUEST_UPDATE"
    }

    private val serviceScope = CoroutineScope(Dispatchers.Default + Job())
    private lateinit var timeTracker: ScreenTimeTracker
    private lateinit var stateManager: ScreenStateManager
    private lateinit var storage: ScreenTimeStorage

    override fun onCreate() {
        super.onCreate()
        Log.d(TAG, "Service en cours de création")

        val notificationManager = getSystemService(Context.NOTIFICATION_SERVICE) as NotificationManager
        val windowManager = getSystemService(Context.WINDOW_SERVICE) as WindowManager

        storage = ScreenTimeStorage(this)

        val screenTimeNotificationManager = ScreenTimeNotificationManager(this, notificationManager)
        val overlayManager = ScreenTimeOverlayManager(this, windowManager)

        timeTracker = ScreenTimeTracker(
            this,
            screenTimeNotificationManager,
            overlayManager,
            storage,
            serviceScope
        )

        stateManager = ScreenStateManager(this, timeTracker, serviceScope)
        stateManager.initialize()

        startForeground(
            ScreenTimeApplication.NOTIFICATION_ID,
            screenTimeNotificationManager.createNotification(0)
        )

        startTimeTracking()
    }

    override fun onStartCommand(intent: Intent?, flags: Int, startId: Int): Int {
        when (intent?.action) {
            ACTION_REQUEST_UPDATE -> timeTracker.broadcastCurrentTime()
        }
        return START_STICKY
    }

    private fun startTimeTracking() {
        serviceScope.launch {
            while (isActive) {
                if (stateManager.isDeviceActive()) {
                    timeTracker.updateTime()
                }
                delay(1000)
            }
        }
    }

    override fun onBind(intent: Intent?): IBinder? = null

    override fun onDestroy() {
        super.onDestroy()
        Log.d(TAG, "Service en cours de destruction")
        timeTracker.cleanup()
        stateManager.cleanup()
        serviceScope.cancel()
    }
}